@@ -2,13 +2,15 @@
 """
 Authentication functions for the SampleDB RESTful API.
 """
+import typing
 
 import flask
 
 from flask_httpauth import HTTPBasicAuth, HTTPTokenAuth, MultiAuth
 
 from ...logic.authentication import login, login_via_api_token, get_active_two_factor_authentication_method
-from ...logic.object_permissions import Permissions
+from ...logic.users import User
+from ...models import Permissions
 from ...utils import object_permissions_required as object_permissions_required_generic
 
 __author__ = 'Florian Rhiem <f.rhiem@fz-juelich.de>'
@@ -19,7 +21,7 @@ multi_auth = MultiAuth(http_basic_auth, http_token_auth)
 
 
 @http_token_auth.verify_token
-def verify_token(api_token):
+def verify_token(api_token: typing.Optional[str]) -> typing.Optional[User]:
     if not api_token:
         return None
     user = login_via_api_token(api_token)
@@ -30,7 +32,7 @@ def verify_token(api_token):
 
 
 @http_basic_auth.verify_password
-def verify_password(username, password):
+def verify_password(username: str, password: str) -> typing.Optional[User]:
     if not username:
         return None
     user = login(username, password)
@@ -38,13 +40,13 @@ def verify_password(username, password):
         return None
     two_factor_authentication_method = get_active_two_factor_authentication_method(user.id)
     if two_factor_authentication_method is not None:
-        # two factor authentication is not supported for the HTTP API
+        # two-factor authentication is not supported for the HTTP API
         return None
     flask.g.user = user
     return user
 
 
-def object_permissions_required(permissions: Permissions):
+def object_permissions_required(permissions: Permissions) -> typing.Callable[[typing.Any], typing.Any]:
     """
     Only allow access to a route it the user has the required permissions.
 
@@ -53,4 +55,9 @@ def object_permissions_required(permissions: Permissions):
 
     :param permissions: the required object permissions
     """
-    return object_permissions_required_generic(permissions, multi_auth, lambda: flask.g.user.id)
+    return object_permissions_required_generic(
+        required_object_permissions=permissions,
+        auth_extension=multi_auth,
+        user_id_callable=lambda: typing.cast(int, flask.g.user.id),
+        may_enable_anonymous_users=False
+    )
