@@ -1,8 +1,10 @@
+import typing
+
 import flask
 
 from . import languages
 
-SUPPORTED_LOCALES = {
+SUPPORTED_LOCALES: typing.Dict[str, typing.Dict[str, str]] = {
     'de': {"english_name": "German", "native_name": "Deutsch"},
     'en': {"english_name": "English", "native_name": "English"}
 }
@@ -20,23 +22,20 @@ def guess_request_locale() -> str:
     """
     allowed_language_codes = get_allowed_language_codes()
     try:
-        best_match_locale = flask.request.accept_languages.best_match(allowed_language_codes)
+        best_match_locale: typing.Optional[str] = flask.request.accept_languages.best_match(allowed_language_codes)
     except Exception:
         return DEFAULT_LOCALE
-    if best_match_locale in SUPPORTED_LOCALES:
+    if best_match_locale is not None and best_match_locale in SUPPORTED_LOCALES:
         return best_match_locale
     else:
         return DEFAULT_LOCALE
 
 
-def get_allowed_language_codes():
+def get_allowed_language_codes() -> typing.List[str]:
     supported_language_codes = list(SUPPORTED_LOCALES.keys())
-    language_codes_allowed = {
-        language.lang_code: language.enabled_for_user_interface
-        for language in languages.get_languages()
-    }
+    allowed_language_codes = languages.get_language_codes(only_enabled_for_user_interface=True)
     return [
         lang_code
         for lang_code in supported_language_codes
-        if language_codes_allowed.get(lang_code, False)
+        if lang_code in allowed_language_codes
     ]
