@@ -3,6 +3,7 @@
 
 """
 import datetime
+import string
 import typing
 
 from . import datatypes
@@ -35,6 +36,14 @@ class Tag(Literal):
         return '<Tag(#{})>'.format(self.value)
 
 
+class Reference(Literal):
+    def __init__(self, input_text: str, start_position: int, value: int) -> None:
+        super(Reference, self).__init__(input_text, start_position, value)
+
+    def __repr__(self) -> str:
+        return '<Reference(#{})>'.format(self.value)
+
+
 class Boolean(Literal):
     def __init__(self, input_text: str, start_position: int, value: datatypes.Boolean) -> None:
         super(Boolean, self).__init__(input_text, start_position, value)
@@ -51,6 +60,14 @@ class Attribute(Literal):
         return '<Attribute({})>'.format(self.value)
 
 
+class Null(Literal):
+    def __init__(self, input_text: str, start_position: int, value: str) -> None:
+        super(Null, self).__init__(input_text, start_position, value)
+
+    def __repr__(self) -> str:
+        return '<Null({})>'.format(self.value)
+
+
 class Quantity(Literal):
     def __init__(self, input_text: str, start_position: int, value: datatypes.Quantity) -> None:
         super(Quantity, self).__init__(input_text, start_position, value)
@@ -95,9 +112,8 @@ class ParseError(Exception):
 
 
 def split_by_texts(tokens: typing.List[typing.Union[Token]]) -> typing.List[typing.Union[Token, Text]]:
-    previous_tokens = tokens
-    tokens = []
-    for token in previous_tokens:
+    new_tokens: typing.List[typing.Union[Token, Text]] = []
+    for token in tokens:
         is_in_text = False
         current_token_start_position = 0
         current_token_text = ""
@@ -106,7 +122,7 @@ def split_by_texts(tokens: typing.List[typing.Union[Token]]) -> typing.List[typi
                 if c == '"':
                     is_in_text = True
                     if current_token_text:
-                        tokens.append(Token(current_token_text, current_token_start_position))
+                        new_tokens.append(Token(current_token_text, current_token_start_position))
                     current_token_text = ''
                     current_token_start_position = i
                     current_token_text += c
@@ -116,7 +132,7 @@ def split_by_texts(tokens: typing.List[typing.Union[Token]]) -> typing.List[typi
                 if c == '"':
                     is_in_text = False
                     current_token_text += c
-                    tokens.append(Text(current_token_text, current_token_start_position, datatypes.Text(current_token_text[1:-1])))
+                    new_tokens.append(Text(current_token_text, current_token_start_position, datatypes.Text(current_token_text[1:-1])))
                     current_token_text = ''
                     current_token_start_position = i + 1
                 else:
@@ -124,20 +140,21 @@ def split_by_texts(tokens: typing.List[typing.Union[Token]]) -> typing.List[typi
         if is_in_text:
             raise ParseError("Unfinished text", current_token_start_position, len(token.input_text))
         if current_token_text:
-            tokens.append(Token(current_token_text, current_token_start_position))
-    return tokens
+            new_tokens.append(Token(current_token_text, current_token_start_position))
+    return new_tokens
 
 
 def split_by_operators(tokens: typing.List[typing.Union[Token, Text]], operators: typing.List[str]) -> typing.List[typing.Union[Token, Text, Operator]]:
+    new_tokens: typing.List[typing.Union[Token, Text, Operator]] = list(tokens)
     for operator in operators:
-        previous_tokens = tokens
-        tokens = []
+        previous_tokens = new_tokens
+        new_tokens = []
         found_operator = True
         while found_operator:
             found_operator = False
             for token in previous_tokens:
                 if found_operator or not isinstance(token, Token):
-                    tokens.append(token)
+                    new_tokens.append(token)
                     continue
                 start_position = token.start_position
                 if ' ' + token.input_text + ' ' == operator:
@@ -157,7 +174,7 @@ def split_by_operators(tokens: typing.List[typing.Union[Token, Text]], operators
                     operator_text = operator
                     before_operator, after_operator = token.input_text.split(operator, 1)
                 else:
-                    tokens.append(Token(token.input_text, start_position))
+                    new_tokens.append(Token(token.input_text, start_position))
                     continue
                 if found_operator:
                     leading_whitespace = len(operator_text) - len(operator_text.lstrip())
@@ -171,125 +188,132 @@ def split_by_operators(tokens: typing.List[typing.Union[Token, Text]], operators
                         operator_text = operator_text[:len(operator_text) - trailing_whitespace]
 
                     if before_operator.strip():
-                        tokens.append(Token(before_operator, start_position))
+                        new_tokens.append(Token(before_operator, start_position))
                     start_position += len(before_operator)
 
-                    tokens.append(Operator(operator_text, start_position, operator_text))
+                    new_tokens.append(Operator(operator_text, start_position, operator_text))
                     start_position += len(operator_text)
 
                     if after_operator.strip():
-                        tokens.append(Token(after_operator, start_position))
+                        new_tokens.append(Token(after_operator, start_position))
                     continue
-            previous_tokens = tokens
-            tokens = []
-        tokens = previous_tokens
-    return tokens
+            previous_tokens = new_tokens
+            new_tokens = []
+        new_tokens = previous_tokens
+    return new_tokens
 
 
-def apply_parentheses(tokens: typing.List[typing.Union[Token, Text, Operator]]) -> typing.List[typing.Union[Token, Text, Operator, list]]:
+def apply_parentheses(tokens: typing.List[typing.Union[Text, Operator, Literal]]) -> typing.List[typing.Union[Text, Operator, Literal, typing.List[typing.Any]]]:
     previous_tokens = tokens
-    tokens = []
-    tokens_stack = [tokens]
+    new_tokens: typing.List[typing.Union[Text, Operator, Literal, typing.List[typing.Any]]] = []
+    tokens_stack = [new_tokens]
     unopened_parentheses_stack = []
     for i, token in enumerate(previous_tokens):
         if not isinstance(token, Operator):
-            tokens.append(token)
+            new_tokens.append(token)
             continue
         if token.operator not in '()':
-            tokens.append(token)
+            new_tokens.append(token)
             continue
         if token.operator == '(':
-            tokens.append([])
-            tokens_stack.append(tokens)
-            tokens = tokens[-1]
+            inner_new_tokens: typing.List[typing.Union[Text, Operator, Literal, typing.List[typing.Any]]] = []
+            new_tokens.append(inner_new_tokens)
+            tokens_stack.append(new_tokens)
+            new_tokens = inner_new_tokens
             unopened_parentheses_stack.append(i)
-            # tokens.append(token)
             continue
         if token.operator == ')':
             if len(tokens_stack) < 2:
                 raise ParseError("Unmatched closing parenthesis", token.start_position, token.start_position + len(token.input_text))
-            # tokens.append(token)
             tokens_stack.pop()
-            tokens = tokens_stack[-1]
+            new_tokens = tokens_stack[-1]
             unopened_parentheses_stack.pop()
             continue
     if unopened_parentheses_stack:
         raise ParseError("Unmatched opening parenthesis", unopened_parentheses_stack[0], unopened_parentheses_stack[0] + 1)
-    return tokens
+    return new_tokens
 
 
-def apply_binary_operator(tokens: typing.List[typing.Union[Token, Text, Operator, list]], operator: str):
+def apply_binary_operator(
+        tokens: typing.List[typing.Union[Text, Operator, Literal, typing.List[typing.Any]]],
+        operator: str
+) -> typing.List[typing.Union[Text, Operator, Literal, typing.List[typing.Any]]]:
     previous_tokens = tokens
-    tokens = []
+    new_tokens: typing.List[typing.Union[Text, Operator, Literal, typing.List[typing.Any]]] = []
     skip_next_token = False
     for i, token in enumerate(previous_tokens):
         if skip_next_token:
             skip_next_token = False
             continue
         if isinstance(token, list):
-            tokens.append(apply_binary_operator(token, operator))
+            new_tokens.append(apply_binary_operator(token, operator))
             continue
         if not isinstance(token, Operator):
-            tokens.append(token)
+            new_tokens.append(token)
             continue
         if token.operator != operator:
-            tokens.append(token)
+            new_tokens.append(token)
             continue
         if token.operator == operator:
-            if not tokens:
+            if not new_tokens:
                 raise ParseError("Binary operator without left operand", token.start_position, token.start_position + len(token.input_text))
-            left_operand = tokens[-1]
-            if not any(isinstance(left_operand, t) for t in (Token, list, Literal)):
+            left_operand = new_tokens[-1]
+            if not isinstance(left_operand, (Token, list, Literal)):
                 raise ParseError("Invalid left operand", left_operand.start_position, token.start_position + len(token.input_text))
-            tokens.pop()
+            new_tokens.pop()
             if not previous_tokens[i + 1:]:
                 raise ParseError("Binary operator without right operand", token.start_position, token.start_position + len(token.input_text))
             right_operand = previous_tokens[i + 1]
-            if not any(isinstance(right_operand, t) for t in (Token, list, Literal)):
+            if not isinstance(right_operand, (Token, list, Literal)):
                 raise ParseError("Invalid right operand", token.start_position, right_operand.start_position + len(right_operand.input_text))
             skip_next_token = True
             expression = [left_operand, token, right_operand]
-            tokens.append(expression)
-    return tokens
+            new_tokens.append(expression)
+    return new_tokens
 
 
-def apply_unary_operator(tokens: typing.List[typing.Union[Token, Text, Operator, list]], operator: str) -> typing.List[typing.Union[Token, Text, Operator, list]]:
+def apply_unary_operator(
+        tokens: typing.List[typing.Union[Text, Operator, Literal, typing.List[typing.Any]]],
+        operator: str
+) -> typing.List[typing.Union[Text, Operator, Literal, typing.List[typing.Any]]]:
     previous_tokens = tokens
-    tokens = []
+    new_tokens: typing.List[typing.Union[Text, Operator, Literal, typing.List[typing.Any]]] = []
     for i, token in reversed(list(enumerate(previous_tokens))):
         if isinstance(token, list):
-            tokens.insert(0, apply_unary_operator(token, operator))
+            new_tokens.insert(0, apply_unary_operator(token, operator))
             continue
         if not isinstance(token, Operator):
-            tokens.insert(0, token)
+            new_tokens.insert(0, token)
             continue
         if token.operator != operator:
-            tokens.insert(0, token)
+            new_tokens.insert(0, token)
             continue
         if token.operator == operator:
-            if not tokens:
+            if not new_tokens:
                 raise ParseError("Unary operator without operand", token.start_position, token.start_position + len(token.input_text))
-            right_operand = tokens[0]
-            if not any(isinstance(right_operand, t) for t in (Token, list, Literal)):
+            right_operand = new_tokens[0]
+            if not isinstance(right_operand, (Token, list, Literal)):
                 raise ParseError("Invalid right operand", token.start_position, right_operand.start_position + len(right_operand.input_text))
             expression = [token, right_operand]
-            tokens[0] = expression
-    return tokens
+            new_tokens[0] = expression
+    return new_tokens
 
 
-def remove_redundant_lists(tokens: typing.List[typing.Union[Token, Text, Operator, list]]) -> typing.List[typing.Union[Token, Text, Operator, list]]:
+def remove_redundant_lists(
+        tokens: typing.List[typing.Union[Text, Operator, Literal, typing.List[typing.Any]]]
+) -> typing.Union[Text, Operator, Literal, typing.List[typing.Union[Text, Operator, Literal, typing.List[typing.Any]]]]:
     if len(tokens) == 1:
         if isinstance(tokens[0], list):
             return remove_redundant_lists(tokens[0])
         return tokens[0]
     previous_tokens = tokens
-    tokens = []
+    new_tokens: typing.List[typing.Union[Text, Operator, Literal, typing.List[typing.Any]]] = []
     for token in previous_tokens:
         if isinstance(token, list):
-            tokens.append(remove_redundant_lists(token))
+            new_tokens.append(remove_redundant_lists(token))
             continue
-        tokens.append(token)
-    return tokens
+        new_tokens.append(token)
+    return new_tokens
 
 
 def parse_date(text: str) -> typing.Optional[datatypes.DateTime]:
@@ -324,16 +348,29 @@ def parse_quantity(text: str, start: int, end: int) -> typing.Optional[datatypes
     if is_negative:
         text = text[1:]
         len_magnitude -= 1
+    if len_magnitude != len(text) and text[len_magnitude].lower() == 'e':
+        for index, character in enumerate(text[len_magnitude + 1:]):
+            len_exponent = index
+            if index == 0 and character == '-':
+                continue
+            if not character.isdigit():
+                break
+        else:
+            len_exponent = len(text[len_magnitude + 1:])
+        if len_exponent > 0:
+            len_magnitude += 1 + len_exponent
 
     if len_magnitude > 0:
+        magnitude: typing.Union[int, float]
         try:
             magnitude = int(text[:len_magnitude])
         except ValueError:
             magnitude = float(text[:len_magnitude])
         if is_negative:
             magnitude = -magnitude
-        units = text[len_magnitude:]
-        units = units.strip()
+        units: typing.Optional[str] = text[len_magnitude:]
+        if units:
+            units = units.strip()
         if not units:
             units = None
         try:
@@ -359,67 +396,121 @@ def parse_tag(text: str) -> typing.Optional[str]:
     return None
 
 
+def parse_reference(text: str) -> typing.Optional[int]:
+    text = text.strip()
+    if text.startswith('#') and all(c in '0123456789' for c in text[1:]):
+        try:
+            value = int(text[1:])
+        except ValueError:
+            return None
+        if str(value) == text[1:]:
+            return value
+    return None
+
+
+def parse_null(text: str) -> typing.Optional[str]:
+    if text.strip().lower() == 'null':
+        return 'null'
+    return None
+
+
 def parse_attribute(text: str, start: int, end: int) -> typing.Optional[typing.List[str]]:
-    if text.strip()[:1] not in 'abcdefghijklmnopqrstuvwxyz':
+    text = text.strip()
+    if text[:1] not in string.ascii_letters:
         return None
-    attributes = text.lower().strip().split('.')
+    attributes = text.split('.')
     for attribute in attributes:
-        if not all(character in 'abcdefghijklmnopqrstuvwxyz0123456789_?' for character in attribute):
+        # empty attributes
+        if not attribute:
             raise ParseError("Invalid attribute name", start, end)
-        if '?' in attribute and attribute != '?':
-            raise ParseError("Invalid array placeholder", start, end)
+        # array placeholder
+        if '?' in attribute:
+            if attribute == '?':
+                continue
+            else:
+                raise ParseError("Invalid array placeholder", start, end)
+        # array indices
+        if attribute[0] in string.digits:
+            if all(character in string.digits for character in attribute):
+                continue
+            else:
+                raise ParseError("Invalid array index", start, end)
+        # attribute name
+        if attribute[0] in string.ascii_letters:
+            if all(character in (string.ascii_letters + string.digits + '_') for character in attribute):
+                continue
+            else:
+                raise ParseError("Invalid attribute name", start, end)
+        raise ParseError("Invalid attribute name", start, end)
     if attributes.count('?') > 1:
         raise ParseError("Multiple array placeholders", start, end)
     return attributes
 
 
-def convert_literals(tokens: typing.List[typing.Union[Token, Text, Operator, list]]) -> typing.List[typing.Union[Operator, Literal, list]]:
+def convert_literals(tokens: typing.List[typing.Union[Token, Text, Operator]]) -> typing.List[typing.Union[Text, Operator, Literal]]:
     previous_tokens = tokens
-    tokens = []
+    new_tokens: typing.List[typing.Union[Text, Operator, Literal]] = []
     for token in previous_tokens:
+        if not isinstance(token, Token):
+            new_tokens.append(token)
+            continue
+
         start = token.start_position
         end = token.start_position + len(token.input_text)
-        if not isinstance(token, Token):
-            tokens.append(token)
+
+        null = parse_null(token.input_text)
+        if null is not None:
+            new_tokens.append(Null(token.input_text, token.start_position, null))
             continue
+
+        reference = parse_reference(token.input_text)
+        if reference is not None:
+            new_tokens.append(Reference(token.input_text, token.start_position, reference))
+            continue
+
         tag = parse_tag(token.input_text)
         if tag is not None:
             accepted_characters = 'abcdefghijklmnopqrstuvwxyz0123456789_-äöüß'
             if not tag or not all(c in accepted_characters for c in tag):
                 raise ParseError("Invalid tag", start + 1, end)
-            tokens.append(Tag(token.input_text, token.start_position, tag))
+            new_tokens.append(Tag(token.input_text, token.start_position, tag))
             continue
+
         boolean = parse_bool(token.input_text)
         if boolean is not None:
-            tokens.append(Boolean(token.input_text, token.start_position, boolean))
+            new_tokens.append(Boolean(token.input_text, token.start_position, boolean))
             continue
+
         date = parse_date(token.input_text)
         if date is not None:
-            tokens.append(Date(token.input_text, token.start_position, date))
+            new_tokens.append(Date(token.input_text, token.start_position, date))
             continue
+
         quantity = parse_quantity(token.input_text, start, end)
         if quantity is not None:
-            tokens.append(Quantity(token.input_text, token.start_position, quantity))
+            new_tokens.append(Quantity(token.input_text, token.start_position, quantity))
             continue
+
         attributes = parse_attribute(token.input_text, start, end)
         if attributes is not None:
-            tokens.append(Attribute(token.input_text, token.start_position, attributes))
+            new_tokens.append(Attribute(token.input_text, token.start_position, attributes))
             continue
+
         raise ParseError("Unable to parse literal", start, end)
-    return tokens
+    return new_tokens
 
 
-def parse_query_string(text: str) -> typing.List[typing.Union[Operator, Literal, list]]:
-    tokens = [Token(text, 0)]
-    tokens = split_by_texts(tokens)
-    tokens = split_by_operators(tokens, ['(', ')', ' in ', '==', '!=', '<=', '>=', '=', '<', '>', ' after ', ' on ', ' before ', '!', ' not ', ' and ', ' or ', ' && ', ' || '])
-    tokens = convert_literals(tokens)
-    tokens = apply_parentheses(tokens)
+def parse_query_string(text: str) -> typing.Union[Text, Operator, Literal, typing.List[typing.Union[Text, Operator, Literal, typing.List[typing.Any]]]]:
+    tokens1: typing.List[Token] = [Token(text, 0)]
+    tokens2: typing.List[typing.Union[Token, Text]] = split_by_texts(tokens1)
+    tokens3: typing.List[typing.Union[Token, Text, Operator]] = split_by_operators(tokens2, ['(', ')', ' in ', '==', '!=', '<=', '>=', '=', '<', '>', ' after ', ' on ', ' before ', '!', ' not ', ' and ', ' or ', ' && ', ' || '])
+    tokens4: typing.List[typing.Union[Text, Operator, Literal]] = convert_literals(tokens3)
+    tokens5: typing.List[typing.Union[Text, Operator, Literal, typing.List[typing.Any]]] = apply_parentheses(tokens4)
     for operator in ['in', '==', '!=', '>=', '<=', '>', '<', 'after', 'on', 'before']:
-        tokens = apply_binary_operator(tokens, operator)
+        tokens5 = apply_binary_operator(tokens5, operator)
     for operator in ['not', '!']:
-        tokens = apply_unary_operator(tokens, operator)
+        tokens5 = apply_unary_operator(tokens5, operator)
     for operator in ['and', '&&', 'or', '||']:
-        tokens = apply_binary_operator(tokens, operator)
-    tokens = remove_redundant_lists(tokens)
-    return tokens
+        tokens5 = apply_binary_operator(tokens5, operator)
+    tokens6: typing.Union[Text, Operator, Literal, typing.List[typing.Union[Text, Operator, Literal, typing.List[typing.Any]]]] = remove_redundant_lists(tokens5)
+    return tokens6
