@@ -7,6 +7,8 @@ import enum
 
 __author__ = 'Florian Rhiem <f.rhiem@fz-juelich.de>'
 
+import typing
+
 
 class Permissions(enum.Enum):
     NONE = 0
@@ -17,17 +19,23 @@ class Permissions(enum.Enum):
     def __contains__(self, item: 'Permissions') -> bool:
         return self.value >= item.value
 
-    def __str__(self):
+    def __str__(self) -> str:
         return self.name.lower()
 
-    def __le__(self, other):
-        return self.value <= other.value
+    def __le__(self, other: 'Permissions') -> bool:
+        if isinstance(other, Permissions):
+            return self.value <= other.value
+        return NotImplemented
 
-    def __lt__(self, other):
-        return self.value < other.value
+    def __lt__(self, other: 'Permissions') -> bool:
+        if isinstance(other, Permissions):
+            return self.value < other.value
+        return NotImplemented
 
     @staticmethod
-    def from_name(name):
+    def from_name(
+            name: str
+    ) -> 'Permissions':
         members = {
             'none': Permissions.NONE,
             'read': Permissions.READ,
@@ -40,7 +48,9 @@ class Permissions(enum.Enum):
             raise ValueError('Invalid name')
 
     @staticmethod
-    def from_value(value):
+    def from_value(
+            value: typing.Optional[int]
+    ) -> 'Permissions':
         if value is None:
             return Permissions.NONE
         for member in Permissions:
